﻿using System;
using System.Threading;

namespace StandardObserver
{
    class ClockTimer : Subject
    {
        private TimeSpan currentTime = TimeSpan.Zero; // default value 00:00:00

        private volatile bool started = false, stopped = true;

        public void SetTime(TimeSpan newTime) { currentTime = newTime; }

        public TimeSpan GetTime() { return currentTime; }

        public void Start()
        {
            if (!started)
            { // but it would be safer using a 'lock'...
                started = true;
                new Thread(Run).Start();
            }
        }

        public void Stop() { stopped = true; }

        private void Run()
        {
            // A thread is either a background thread or a foreground thread.
            // Background threads are identical to foreground threads, except
            // that background threads do not prevent a process from terminating.
            // Once all foreground threads belonging to a process have terminated,
            // the common language runtime ends the process. Any remaining 
            // background threads are stopped and do not complete.
            // https://msdn.microsoft.com/en-us/library/system.threading.thread.isbackground%28v=vs.110%29.aspx
            Thread.CurrentThread.IsBackground = true;

            stopped = false;
            while (!stopped)
            {
                Thread.Sleep(1000); // pause : 1000 msec == 1 sec
                Tick();
            }
            started = false; stopped = true;
        }

        private void Tick()
        {
            currentTime += new TimeSpan(0, 0, 1); // increment: 1 sec
            Notify();
        }

    }
}
